@@ -2,17 +2,11 @@ use crate::input::Input;
 use std::collections::HashSet;
 use std::hash::{BuildHasherDefault, Hasher};
 
+#[derive(Default)]
 struct CustomHash {
     hash: u64,
 }
 
-impl Default for CustomHash {
-    #[inline]
-    fn default() -> Self {
-        Self { hash: 0 }
-    }
-}
-
 impl Hasher for CustomHash {
     fn finish(&self) -> u64 {
         self.hash as u64
